#include "utils.h"

#include <string.h>
#include <stdarg.h>

/**
* \brief Computes the length required to store the path generated by #ch5_util_join_path
* \param[in] num_parts The number of parts to glue together (should match the
*                      numer of params after \p ext)
* \param[in] glue      Pointer to a single char which will be used to "glue"
*                      the path segments together
* \param[in] ext       Optional pointer to a string that acts as the extension
*                      of the path (eg. \p ".jpg") - note that the period must
*                      be included in the string.  Pass \p NULL for no
*                      extension.
* \param[in] ...       Variable list of strings that are the path fragments to
*                      be "glued" together.
* \returns The length of a string required to store the joined path
*/
int ch5_util_join_path_strlen(int num_parts, const char *glue, const char *ext, ...) {
  int length = num_parts;/* This also takes care of counting the null terminator */
  va_list vargs;
  va_start(vargs, ext);
  for (int i = 0; i < num_parts; i++) {
    char *x = va_arg(vargs, char*);
    length += strlen(x);
  }
  if (ext != NULL) length += strlen(ext);
  va_end(vargs);
  return length;
}

/**
* \brief Joins a series of path segments together into a single path
* \pre See \p out parameter details
* \param[out] out      Pointer to a string whose capacity is at least that of
*                      the result of #ch5_util_join_path_strlen with the same
*                      arguments for \p num_parts, \p glue and \p ext.
* \param[in] num_parts The number of parts to glue together (should match the
*                      numer of params after \p ext)
* \param[in] glue      Pointer to a single char which will be used to "glue"
*                      the path segments together
* \param[in] ext       Optional pointer to a string that acts as the extension
*                      of the path (eg. \p ".jpg") - note that the period must
*                      be included in the string.  Pass \p NULL for no
*                      extension.
* \param[in] ...       Variable list of strings that are the path fragments to
*                      be "glued" together.
*/
void ch5_util_join_path(char *out, int num_parts, const char *glue, const char *ext, ...) {
  va_list vargs;
  va_start(vargs, ext);
  char *index = out;
  for (int i = 0; i < num_parts; i++) {
    char *x = va_arg(vargs, char*);
    strcpy(index, x);
    index += strlen(x);
    if (i < num_parts - 1) {
      strcpy(index, glue);
      index += strlen(glue);
    }
  }
  if (ext != NULL) strcpy(index, ext);
  va_end(vargs);
}

int ch5_int_min(int a, int b) {
  return (a < b) ? a : b;
}
