/** This class implements an auxilliary grid which can be used to present 
 *  time varying (or fixed) data. The auxilliary grid itself can be totally dynamic,
 *  meaning that points and the elements derived from them can change. If both the
 *  grid and data are dynamic, there must be the same number of time instances. 
 *
 */
#ifndef AUXGRID_H
#define AUXGRID_H

#include <vector>
#include "render_utils.h"
#include "Model.h"
#include "DrawingObjects.h"
#include "plottingwin.h"
#include "DeadDataGUI.h"

class TBmeshWin;

class AuxGridFetcher;

class AuxGrid {
    private:
        AuxGridFetcher * _indexer = NULL;  //!< private implementation of indexer
        bool         _display   = true;
        bool         _datafied[maxobject];
        bool         _surf_fill = true;  //!< if false, draw outline
        bool         _vol_fill  = false;   //!< if false, draw wireframe
        bool         _autocol=false;
        bool         _show[maxobject]; //!< true to show object
        bool         _3D[maxobject+2];
        GLfloat      _color[maxobject+2][4];
        float        _size[maxobject+2];
        bool         _plottable = false;    //!< true if a time series can be plotted
        int          _hiVert    = 0;
        bool         _hilight   = false;
        PlotWin*     _timeplot  = NULL;
        int          _sz_ts;               //!< size of time series
        double*      _time_series;
        bool         _clip      = false;   // is is clipped?
        float        _t0;
        float        _dt;
        void         fill_buff( void *, Model *m, int );
        RenderTris    _rd_vol;
        RenderTris    _rd_surf;
        RenderLines   _rd_lns;
        RenderSpheres _rd_pts;
        GLfloat       _mat_prop[4] = { 0.7, 0.7, 80, 1. };
        string        _absfile;
    public:
        AuxGrid( const char *fn, TBmeshWin *, AuxGrid *ag=NULL, float t0=0, float dt=1 );
        ~AuxGrid();

        Colourscale cs;
        void     draw( int, unsigned int, void* );
        void     color( Object_t o, GLfloat *r );
        GLfloat * color( Object_t o ){ return _color[o]; }
        bool     showobj( Object_t o, bool b ) { bool old=_show[o];_show[o]=b;return old;}
        bool     showobj( Object_t o ) { return _show[o];}
        void     threeD( Object_t o, bool b ) { _3D[o] = b; }
        bool     threeD( Object_t o ) const { return _3D[o]; }
        void     size( Object_t o, float s ) { _size[o] = s; }
        float    size( Object_t o ) const {return _size[o]; }
        int      num_tm();
        void     display( bool b ){ _display=b; }
        void     datify( Object_t o, bool b ){ _datafied[o]=b; }
        void     surfill( bool b ){ _surf_fill=b; }
        void     volfill( bool b ){ _vol_fill=b; }
        void     autocolor( bool b ){ _autocol=b; }
        void     optimize_cs(int);
        bool     highlight_vertex(int n, float &val, bool update_plot=true);
        void     highlight(bool b){_hilight=b;}
        void     plot(int);
        bool     plottable(){return _plottable;}
        bool     data();
        int      num_vert();
        void     clip( bool b ){ _clip=b; }
        void     set_mat( GLfloat *mp, int s=0 ) { memcpy( _mat_prop, mp, 4*sizeof(GLfloat) ); 
               _rd_vol.set_material(s,_mat_prop); _rd_surf.set_material(s,_mat_prop);
               _rd_pts.set_material(s,_mat_prop );_rd_lns.set_material(s,_mat_prop); }
        GLfloat *get_mat() { return _mat_prop; }
        string   file(){ return _absfile; }
        DeadDataGUI*  _deadData  = NULL;
};

#endif
