#include "CutSurfaces.h"
#include <string.h>
#include "VecData.h"
#include <stdlib.h>

const int ADD_INC = 1000;

CutSurfaces::CutSurfaces(GLfloat *n) 
{
  fillcolor( 1., 0, 0.1 );
  outlinecolor( 0.125, 0.8, 0.7 );
  memcpy( _norm, n, 3*sizeof(GLfloat) );
  _rd.set_material( 0, (const GLfloat[]){0,0,0,1} );
  _rd.ambient = 1.;
  _rd.back_lit = 1.;
}


/** add a Surface element to the surface
 *
 *  \param se the element
 *  \param n  the normal
 *  \param ni list for data interpolation
 *
 *  \pre each element pointer points to a single element and not a list
 *
 *  \note points are not shared between elements
 */
void
CutSurfaces :: addEle( SurfaceElement *se, Interpolator<DATA_TYPE> *ni )
{
  _ele.push_back(se);
  _ptarr.push_back(const_cast<GLfloat *>(const_cast<PPoint*>(se->pt())->pt()));
  _interp.push_back(ni);
}


/** assume that since everything is temporary, it will be our perogative
 * to destroy it all
 */
CutSurfaces::~CutSurfaces()
{
  for( auto &e : _ele )
    delete e;
  for( auto &p : _ptarr )
    delete p;
  for( auto &i : _interp )
    delete[] i;
}


void 
CutSurfaces::clear( GLfloat *n )
{
  if ( !_ele.empty() ) 
    for ( int i=0; i<_ele.size(); i++ ) {
      delete _ele[i];
      //delete _ptarr[i];
      delete[] _interp[i];
    }
  _ele.clear();
  _ptarr.clear();
  _interp.clear();
  _zlist.clear();
  _rd.clear(); 
  _p = NULL;
  if( n ) memcpy( _norm, n, 3*sizeof(GLfloat) );
  _fresh = true;
}


void
/**
 * @brief recolour surface 
 *
 * @param col        cut surface
 * @param showdata   whether surface is datified
 */
CutSurfaces::recolor( const GLfloat *col, bool showdata )
{
  GLfloat *buff = _rd.buffer();
  for( int i=0; i<_rd.numVtx(); i++ ){
    if( showdata ) 
      buff[VBO_COLOR_OFFSET+VBO_ELEM_SZ*i] = col[3];
    else
      memcpy( buff+VBO_COLOR_OFFSET+VBO_ELEM_SZ*i, col, 4*sizeof(GLfloat) );
  }
  _rd.update_nodalbuff();
}

