#ifndef DATACLASS_H
#define DATACLASS_H

#include <zlib.h>
#include "IGBheader.h"
#include <map>
#include <string>
#include <exception>

using namespace std;

enum DataReaderEnum {AllInMem, Threaded };
enum fileType { FTIGB=1, FTascii, FTfileSeqCG, FTDynPt, FThdf5, FTother };

fileType FileTypeFinder ( const char *fn );
void CG_file_list( map<int,string>&filelist, const char *fn );
int  parse_HDF5_grid( const char *fn, string& type, unsigned int& index );

class FrameMismatch
{
    public:
        int expected, got;
        FrameMismatch( int a, int b ){expected=a;got=b;}
};

class PointMismatch
{
    public:
        int expected, got;
        PointMismatch( int a, int b ){expected=a;got=b;}
};


/** The basic class for reading data */
template<class T>
class DataClass
{
  public:
    virtual T      max(int)=0;	     //!< maximum data value at a time
    virtual T      max()=0;	         //!< maximum data value at a time
    virtual T      min(int)=0;       //!< minimum data value at a time
    virtual T      min()=0;          //!< minimum data value at a time
    virtual T*     slice(int)=0;     //!< pointer to time slice of data
    virtual void   time_series( int, T* )=0; //!< time series for a point
    virtual void   increment(int)=0;         //!< time slice increment
    int      max_tm(){return maxtm;}          //!< maximum allowable time
    int      slice_sz(){return slice_size;}   //!< size of 1 time slice
    void     slice_sz(int a){slice_size=a;}   //!< set size of slice
    string   file(){return filename;}         //!< return the file name
    float    t0(void){ return _t0; }          //!< initial time read
    float    dt(void){ return _dt; }          //!< time increment
    fileType filetype(void){ return _ftype; } //!< type of file

    DataClass(){}
    virtual ~DataClass(){}

  protected:
    T*       data=NULL;    //!< data
    int      maxtm=0;      //!< number of time slice
    int      last_tm=-1;   //!< last time that can be requested
    int      slice_size=0; //!< amountof data in one time slice
    float    _dt=1;        //!< time increment
    float    _t0=0;        //!< initial time read in
    string   filename;     //!< file containing data
    fileType _ftype;       //!< file type
};

#endif
