#include "DrawingObjects.h"

const int PolyGon::_zero=0;

/** fill buffer with one Polygon 
 *
 *  \param p0       index of first point to draw
 *  \param c        colour to use if no data
 *  \param cs       colour scale
 *  \param data     data associated with elements if flat, else nodes (NULL for no data display)
 *  \param dataopac data opacity
 *  \param nrml     if flat element normal, else vertex normals (NULL for none)
 *  \param buffer   buffer entry in which to place data
 *  \param surf     surface we are in 
 *  \param flat     flat shading
 *
 *  \return number of triangles put in draw buffer
 */
int PolyGon::buffer( int p0, GLfloat *c, Colourscale* cs,
                          DATA_TYPE* data, dataOpac* dataopac,
                          const GLfloat* nrml, GLfloat *buffer, 
                          int surf, bool flat )
{
  int i=p0*4;
  for ( int j=0; j<_ptsPerObj; j++ )
    if ( !_pt->vis(_node[i+j]) )
      return 0;

  for( int n=0; n<_ptsPerObj-2; n++ ) 
    for( int j=0; j<3; j++ ) {
      GLfloat *buff_off = buffer + (j+n*3)*VBO_ELEM_SZ; 

      int k = j==2 ? 0 : n+1+j;

      const GLfloat *col = colour( flat?p0:_node[i+k], c, data, cs, dataopac );  
      buff_vtx( buff_off, _pt->pt(_node[i+k]), nrml+(flat?0:_node[i+k]*3), col, surf );
    }
  return _ptsPerObj-2;
}


/** for a surface with ordered node numbers, compute the normals
 *
 * \param e0 first element
 * \param e1 last element
 */
void PolyGon::compute_normals(int e0, int e1)
{
  int e, n;

  if ( _nrml==NULL ) _nrml=new GLfloat[e1*3+3];

  for ( e=e0; e<=e1; e++ ) {

    const int* ele = _node+3*e;
    Vector3D<GLfloat> a(_pt->pt(ele[0]));
    Vector3D<GLfloat> b(_pt->pt(ele[1]));
    Vector3D<GLfloat> c(_pt->pt(ele[2]));

    b -= a;
    b = b.Cross(c-a);
    b.Normalize();
    _nrml[3*e]   = b.X();
    _nrml[3*e+1] = b.Y();
    _nrml[3*e+2] = b.Z();
  }
}

